@@ -1,7 +1,7 @@
 """
 unmatcher :: Regular expression reverser for Python
 """
-__version__ = "0.0.1"
+__version__ = "0.0.2"
 __author__ = "Karol Kuczmarski"
 __license__ = "Simplified BSD"
 
@@ -9,6 +9,7 @@ __license__ = "Simplified BSD"
 import random
 import re
 import string
+import sys
 
 
 __all__ = ['reverse']
@@ -24,7 +25,7 @@ def reverse(pattern, *args, **kwargs):
 
     :return: String that matches ``pattern``
     """
-    if isinstance(pattern, basestring):
+    if is_string(pattern):
         flags = None
     else:
         # assuming regex object
@@ -43,12 +44,17 @@ def reverse(pattern, *args, **kwargs):
         groupvals[i] = value
     groups = resolve_groupvals(sre_subpattern.pattern, groupvals)
 
-    reversal = Reversal(sre_subpattern.data, flags=flags, groups=groups)
+    reversal = Reversal(sre_subpattern.data, flags=flags, groups=groups,
+                        string_class=type(pattern))
     return reversal.perform()
 
 
 # Implementation
 
+IS_PY3 = sys.version[0] == '3'
+is_string = lambda x: isinstance(x, (str if IS_PY3 else basestring))
+
+
 def resolve_groupvals(sre_pattern, groupvals):
     """Resolve a dictionary of capture group values (mapped from either
     their names or indices), returning an array of those values ("mapped" only
@@ -61,14 +67,12 @@ def resolve_groupvals(sre_pattern, groupvals):
     group_count = sre_pattern.groups
     names2indices = sre_pattern.groupdict
 
-    # TODO: detect surplus keys in ``groupvals`` that do not match
-    # any actual capture group inside ``sre_pattern``
     groups = [None] * group_count
     for ref, value in groupvals.iteritems():
-        index = names2indices.get(ref, ref)
         try:
+            index = names2indices[ref] if is_string(ref) else ref
             groups[index] = value
-        except (TypeError, IndexError):
+        except (IndexError, KeyError, TypeError):
             raise ValueError("invalid capture group reference: %s" % ref)
 
     return groups
@@ -77,6 +81,7 @@ def resolve_groupvals(sre_pattern, groupvals):
 class Reversal(object):
     """Encapsulates the reversal process of a single regular expression."""
 
+    # TODO: choose among Unicode characters if using Unicode
     BUILTIN_CHARSETS = {
         'word': string.ascii_letters,
         'digit': string.digits,
@@ -84,7 +89,7 @@ class Reversal(object):
     }
     MAX_REPEAT = 64
 
-    def __init__(self, regex_ast, flags=None, groups=None):
+    def __init__(self, regex_ast, flags=None, groups=None, string_class=None):
         """Constructor.
 
         Use keywords to pass arguments other than ``regex_ast``.
@@ -93,6 +98,12 @@ class Reversal(object):
         self.flags = flags or 0
         self.groups = groups or [None]
 
+        # use correct string class depending on Python version or argument
+        if string_class is None:
+            string_class = str if IS_PY3 else unicode
+        self._str = string_class
+        self._chr = unichr if string_class.__name__ == 'unicode' else chr
+
     def perform(self):
         return self._reverse_nodes(self.regex_ast)
 
@@ -102,13 +113,12 @@ class Reversal(object):
         """Generates string matching given sequence of nodes
         from regular expressions' abstract syntax tree (AST).
         """
-        # TODO: preserve unicodeness/ANSIness in Python 2
-        return ''.join(map(self._reverse_node, nodes))
+        return self._str().join(map(self._reverse_node, nodes))
 
     def _reverse_node(self, (type_, data)):
         """Generates string matching given node from regular expression AST."""
         if type_ == 'literal':
-            return chr(data)
+            return self._chr(data)
         if type_ == 'any':
             return random.choice(self._charset('any'))
 
@@ -157,7 +167,7 @@ class Reversal(object):
         # range (e.g. a-z) inside [ ]
         if type_ == 'range':
             min_char, max_char = data
-            return chr(random.randint(min_char, max_char))
+            return self._chr(random.randint(min_char, max_char))
 
         # built-in character set: \d, \w, etc.
         if type_ == 'category':
@@ -248,10 +258,10 @@ class Reversal(object):
             if flags & re.DOTALL:
                 return string.printable
             else:
-                visible_chars = ''.join(
+                visible_chars = self._str().join(
                     v for k, v in self.BUILTIN_CHARSETS.iteritems()
                     if k != 'space')
-                return visible_chars + ' '
+                return visible_chars + self._str(" ")
 
         if name in self.BUILTIN_CHARSETS:
             return self.BUILTIN_CHARSETS[name]
